unit LCD;
{
  LCD Unit for MPXF
  Written by Scott Bradshaw
}

interface

procedure ReadLCDSettings;
procedure LCDReady;
procedure LCDCleanUp;
procedure LCDCursorOn;
procedure LCDCursorOff;
procedure LCDSetPort(p:word);
procedure LCDSetLines(l:byte);
procedure LCDSetLength(l:byte);
procedure LCDHome;
procedure LCDClear;
procedure LCDChar(ch:char);
procedure LCDPrint(s:string);
procedure DrawMPXFLogo;
procedure LCDPrintAt(l:byte;s:string);
procedure LCDCursor(l,x:byte);
procedure LCDToggle;
procedure InitLCD;
procedure LCDDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
procedure LCDWriteData(b:byte);


procedure lcdchara(b:byte);

function Lines:word;
function Width:word;
function Registered:boolean;
function GetLCDType:word;

var
  LCDtoScreen:boolean;
  RegName,RegCode:string;
  const version =  '1.4';

implementation

uses newdelay,crt,
 {$IFDEF parallel} pcomlcd,lptlcd, {$ENDIF,}config,
 {$IFDEF serial} comlcd,{$ENDIF}dos
 {$IFDEF other} ,gbcomm,vfdlcd {$ENDIF}
 {$IFDEF pic}, linkp {$ENDIF};
var

    lcdlines:byte;
    lcdlength:byte;
    lcdtype:byte;
    controlor:byte;

function Registered:boolean;
 begin
    registered := true;
{  if calcregcode(regname) = RegCode then registered:=true else registered:=false;}
 end;

function GetLCDType:word;
  begin GetLCDType := LCDType;
  end;
procedure GetaTime(var Hour, Minute, Second, Sec100 : byte); assembler;
{Procedure GetTime; assembler;
{ GETTIME - DOS service function
  Description: Returns the current time set in the operating system.
               Ranges of the values returned are: Hour 0-23, Minute 0-59,
               Second 0-59 and Sec100 (hundredths of seconds) 0-99 }
Asm
  MOV AH,2CH
  INT DOS
  XOR AH,AH
  MOV AL,DL
  LES DI,Sec100
  STOSB
  MOV AL,DH
  LES DI,Second
  STOSB
  MOV AL,CL
  LES DI,Minute
  STOSB
  MOV AL,CH
  LES DI,Hour
  STOSB
End; { GetTime }

procedure DrawMPXFLogo;
var b:byte;
    sx,sy:byte;
    h,m,s,ss:word;
    ts:string;
    a:string;
    start:byte;
  begin
    start := 8;
    sx:=wherex; sy := wherey;
    gotoxy(1,start);
    gettime(h,m,s,ss);
    str(h,a);
    if length(a) = 1 then a:='0'+a;
    ts := a+':';
    str(m,a);
    if length(a) = 1 then a:='0'+a;
    ts := ts + a + ':';
    str(s,a);
    if length(a) = 1 then a:='0'+a;
    ts := ts + a;
    for b := 1 to 80 do
      write('');
    gotoxy(1,start+1);
    write('  \ /    ');
    write(lcdlines,'x',lcdlength,' LCD connected to ');
    if lcdtype = 1 then write('Parallel Port') else
    if lcdtype = 2 then write('Serial Port') else
    if lcdtype = 3 then write('Gameboy') else
    if lcdtype = 4 then write('VFD') else
    if lcdtype = 5 then write('PICanLCD') else
    if lcdtype = 6 then write('Serial-Parallel');
    gotoxy(1,start+2);
    write('   *     Written by Scott Bradshaw');gotoxy(1,start+3);
    write('    / \     Version ',version);
    if Registered then
      write('     Registered to: ',regname) else write(' UNREGISTERED!');
    gotoxy(1,start+5);
    for b := 1 to 34 do
      write('');
    write(ts);
    for b := 1 to 38 do
      write('');
    gotoxy(sx,sy);

  end;

procedure Status(s:string);
  begin
  end;

function Lines:word; begin lines:=lcdlines; end;
function width:word; begin width:=lcdlength; end;
procedure LCDSetLines(l:byte); begin LCDLines := l; end;
procedure LCDSetLength(l:byte); begin LCDLength := l; end;

procedure LCDDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
  begin
    case LCDType of
      0: begin end;
      {$IFDEF serial} 2:SDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8); {$ENDIF}
      {$IFDEF parallel}1:LPTDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8);{$ENDIF}
      {$IFDEF other} 4:VFDDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8); {$ENDIF}
      {$IFDEF pic } 5:PICDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8); {$ENDIF}
      {$IFDEF parallel}6: PSDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8);{$ENDIF}
    end;
  end;

procedure LCDWriteData(b:byte);
  begin
    {$IFDEF parallel}LPTPushIns(b);{$ENDIF}
  end;

procedure LCDReady;
 begin
 {$IFDEF serial}    if LCDType = 2 then SInitLCD; {$ENDIF}
 end;

procedure LCDChar(ch:char);
  begin
    case LCDType of
      0: begin end;
{$IFDEF parallel}      1: LPTPushChar(ord(ch)); {$ENDIF}
 {$IFDEF serial}       2: SPushChar(ch); {$ENDIF}
 {$IFDEF other}       3: print_char(ch);
      4: VFDPushChar(ord(ch)); {$ENDIF}
      {$IFDEF pic} 5: PICPushchar(ord(ch)); {$ENDIF}
      {$IFDEF parallel} 6: PSPushchar(ch); {$ENDIF}
    end;
  end;

procedure LCDCleanUp;
  begin
 {$IFDEF serial}     if LCDType = 2 then SClose; {$ENDIF}
  end;

procedure LCDPrint(s:string);
  var i:integer;
  begin
    if LCDtoScreen then begin
     write(s);
    end;
{   if lcdtype = 5 then PICPrintLCD(s) else}
   begin
     s := copy(s,1,lcdlength);
     for i := 1 to length(s) do LCDChar(s[i]);
   end;
  end;

 procedure LCDCursorOn;
   begin
     case LCDType of
       0: begin end;
{$IFDEF parallel}       1: LPTCursorOn; {$ENDIF}
 {$IFDEF serial}        2: SCursorOn; {$ENDIF}
 {$IFDEF other}        4: VFDCursorOn; {$ENDIF}
 {$IFDEF pic}          5: PICCursorOn; {$ENDIF}
 {$IFDEF parallel}       6: PSCursorOn; {$ENDIF}
     end;
   end;

 procedure LCDCursorOff;
   begin
     case LCDType of
       0:begin end;
     {$IFDEF parallel}  1: LPTCursorOff; {$ENDIF}
 {$IFDEF serial}        2: SCursorOff; {$ENDIF}
 {$IFDEF other}        4: VFDCursorOff; {$ENDIF}
  {$IFDEF pic}          5: PICCursorOn; {$ENDIF}
{$IFDEF parallel}       6: PSCursorOn; {$ENDIF}
     end;
   end;

procedure LCDHome;
  begin
    if LCDtoScreen then  gotoxy(1,1);
    case LCDType of
      0: begin end;
{$IFDEF parallel}      1: LPTHome; {$ENDIF}
 {$IFDEF serial}       2: SHome; {$ENDIF}
 {$IFDEF other}       4: VFDHome; {$ENDIF}
  {$IFDEF pic}          5: PICClearLCD; {$ENDIF}
{$IFDEF parallel}       6: PSHome; {$ENDIF}
    end;
  end;

procedure LCDToggle; begin {$IFDEF parallel}LPTToggle;{$ENDIF} end;


procedure LCDClear;
  begin
    case LCDType of
      0: begin end;
{$IFDEF parallel}      1: LPTClear; {$ENDIF}
 {$IFDEF serial}       2: SClear; {$ENDIF}
 {$IFDEF other}       3: clear_scr;
      4: VFDClear; {$ENDIF}
 {$IFDEF pic}          5: PICClearLCD; {$ENDIF}
 {$IFDEF parallel}     6: PSClear; {$ENDIF}
    end;
    if LCDtoScreen then begin
      clrscr;
      drawmpxflogo;
    end;
  end;

procedure LCDSetPort(p:word); begin {$IFDEF parallel}LPTsetport(p); {$ENDIF} end;


procedure InitLCD;
  begin
    if (lcdtype = 0) then lcdtype:=0;
    case lcdtype of
      0: begin end;
{$IFDEF parallel}      1: LPTInitLCD; {$ENDIF}
 {$IFDEF serial}       2: SInitLCD; {$ENDIF}
 {$IFDEF other}       4: VFDInitLCD; {$ENDIF}
   {$IFDEF pic}          5: PICInit; {$ENDIF}
{$IFDEF parallel}      6: PSInitLCD; {$ENDIF}
    end;
    delay(20);
  end;


procedure LCDPrintAt(l:byte;s:string);
  var i:integer;
  begin
   {$IFDEF other}
    if LCDType = 3 then
    begin
      if lcdtoscreen then gotoxy(((80-lcdlength) div 2) + 1,l);
      write(s);
      goto_line(l);
      print_line(s);
{      if lcdtoscreen then gotoxy(((80-lcdlength) div 2) + 1,l);}
    end else {$ENDIF}
    begin
      LCDCursor(l,1);
      LCDPrint(s);
    end;
  end;

procedure LCDCursor(l,x:byte);
  begin
    if l <= lcdlines then
    begin
      if lcdtoscreen then gotoxy(((80-lcdlength) div 2) + x,l);
      case LCDType of
        0: begin end;
  {$IFDEF parallel}      1: LPTCursor(l,x); {$ENDIF}
   {$IFDEF serial}       2: SCursor(l,x,lcdlength); {$ENDIF}
   {$IFDEF other}       3: goto_xy(x,l);
        4: VFDCursor(l,x,lcdlength); {$ENDIF}
          {$IFDEF pic}          5: PICGotoXY(l,x); {$ENDIF}
  {$IFDEF parallel}      6: PSCursor(l,x,lcdlength); {$ENDIF}
      end;
   end;
  end;

procedure lcdchara(b:byte);
  begin lcdchar(chr(b));
  end;

procedure ReadLCDSettings;
 var cs:string;
  begin
    if not (openconfig('mpxflcd.cfg')) then begin
      writeln('Error opening LCD config file: mpxflcd.cfg!');
      halt(1);
    end;
    RegName := ReadString('Registration','Name');
    RegCode := ReadString('Registration','RegCode');
    LCDLines := ReadWord('LCD Settings','lcdlines');
    LCDLength := ReadWord('LCD Settings','lcdlength');
    cs := ReadString('LCD Settings','LCD');
    LCDType := ReadWord(cs,'LCDType');
    case LCDType of
      0: begin end;
{$IFDEF parallel}      1: begin
           LPTReadSettings(cs);
           if ((LCDLines = 4) and (LCDLength = 40)) then
            else SetE2Pin(0);
         end; {$ENDIF}
 {$IFDEF serial}      2: SerialReadSettings(cs); {$ENDIF}
 {$IFDEF other}      4: VFDReadSettings(cs); {$ENDIF}
   {$IFDEF pic}          5: PICReadSettings(cs); {$ENDIF}
    {$IFDEF parallel}      6: PSerialReadSettings(cs); {$ENDIF}
    end;
    closeconfig;
  end;




begin
{  chardelay := 1;
  controldelay := 2;
  LCDtoScreen := True;
  Portaddress := 888; { $378 }
{  InitLCD; Don't Init, since we don't know the # of lines yet }
end.

